@@ -43,7 +43,19 @@ __protobuf__ = proto.module(
 
 
 class RouteMatrixElementCondition(proto.Enum):
-    r"""The condition of the route being returned."""
+    r"""The condition of the route being returned.
+
+    Values:
+        ROUTE_MATRIX_ELEMENT_CONDITION_UNSPECIFIED (0):
+            Only used when the ``status`` of the element is not OK.
+        ROUTE_EXISTS (1):
+            A route was found, and the corresponding
+            information was filled out for the element.
+        ROUTE_NOT_FOUND (2):
+            No route could be found. Fields containing route
+            information, such as ``distance_meters`` or ``duration``,
+            will not be filled out in the element.
+    """
     ROUTE_MATRIX_ELEMENT_CONDITION_UNSPECIFIED = 0
     ROUTE_EXISTS = 1
     ROUTE_NOT_FOUND = 2
@@ -114,13 +126,49 @@ class ComputeRoutesRequest(proto.Message):
             FUEL_EFFICIENT reference route calculation takes into
             account various parameters that would generate an optimal
             fuel efficient route.
+        extra_computations (MutableSequence[google.maps.routing_v2.types.ComputeRoutesRequest.ExtraComputation]):
+            Optional. A list of extra computations which
+            may be used to complete the request. Note: These
+            extra computations may return extra fields on
+            the response. These extra fields must also be
+            specified in the field mask to be returned in
+            the response.
     """
 
     class ReferenceRoute(proto.Enum):
-        r"""A supported reference route on the ComputeRoutesRequest."""
+        r"""A supported reference route on the ComputeRoutesRequest.
+
+        Values:
+            REFERENCE_ROUTE_UNSPECIFIED (0):
+                Not used. Requests containing this value
+                fail.
+            FUEL_EFFICIENT (1):
+                Fuel efficient route. Routes labeled with
+                this value are determined to be optimized for
+                parameters such as fuel consumption.
+        """
         REFERENCE_ROUTE_UNSPECIFIED = 0
         FUEL_EFFICIENT = 1
 
+    class ExtraComputation(proto.Enum):
+        r"""Extra computations to perform while completing the request.
+
+        Values:
+            EXTRA_COMPUTATION_UNSPECIFIED (0):
+                Not used. Requests containing this value will
+                fail.
+            TOLLS (1):
+                Toll information for the route(s).
+            FUEL_CONSUMPTION (2):
+                Estimated fuel consumption for the route(s).
+            TRAFFIC_ON_POLYLINE (3):
+                Traffic aware polylines for the route(s).
+        """
+        EXTRA_COMPUTATION_UNSPECIFIED = 0
+        TOLLS = 1
+        FUEL_CONSUMPTION = 2
+        TRAFFIC_ON_POLYLINE = 3
+
     origin: gmr_waypoint.Waypoint = proto.Field(
         proto.MESSAGE,
         number=1,
@@ -184,6 +232,11 @@ class ComputeRoutesRequest(proto.Message):
         number=14,
         enum=ReferenceRoute,
     )
+    extra_computations: MutableSequence[ExtraComputation] = proto.RepeatedField(
+        proto.ENUM,
+        number=15,
+        enum=ExtraComputation,
+    )
 
 
 class ComputeRoutesResponse(proto.Message):
@@ -253,8 +306,28 @@ class ComputeRouteMatrixRequest(proto.Message):
             you made the request. If you set this value to a
             time that has already occurred, the request
             fails.
+        extra_computations (MutableSequence[google.maps.routing_v2.types.ComputeRouteMatrixRequest.ExtraComputation]):
+            Optional. A list of extra computations which
+            may be used to complete the request. Note: These
+            extra computations may return extra fields on
+            the response. These extra fields must also be
+            specified in the field mask to be returned in
+            the response.
     """
 
+    class ExtraComputation(proto.Enum):
+        r"""Extra computations to perform while completing the request.
+
+        Values:
+            EXTRA_COMPUTATION_UNSPECIFIED (0):
+                Not used. Requests containing this value will
+                fail.
+            TOLLS (1):
+                Toll information for the matrix element(s).
+        """
+        EXTRA_COMPUTATION_UNSPECIFIED = 0
+        TOLLS = 1
+
     origins: MutableSequence["RouteMatrixOrigin"] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
@@ -280,6 +353,11 @@ class ComputeRouteMatrixRequest(proto.Message):
         number=5,
         message=timestamp_pb2.Timestamp,
     )
+    extra_computations: MutableSequence[ExtraComputation] = proto.RepeatedField(
+        proto.ENUM,
+        number=8,
+        enum=ExtraComputation,
+    )
 
 
 class RouteMatrixOrigin(proto.Message):
